@@ -1,16 +1,18 @@
-__version__ = "0.2.0"
+__version__ = "0.3.0"
 
 from .symbol import Symbol, register_type
 from .expression import Expr, Head, symbol
-from .macro import Macro
+from .macro import Macro, MacroFlags, MacroFlagOptions
 from .ast import parse
 from functools import wraps
 
 __all__ = ["Symbol", "Head", "Expr", "Macro",
            "symbol", "register_type", "parse",
-           "blocked", "record", "property", "dump", "get_macro"
+           "blocked", "record", "property", "dump", "get_macro", "set_flags"
            ]
 
+# global macro instance and its functions
+
 _MACRO = Macro()
 
 @wraps(_MACRO.blocked)
@@ -29,7 +31,18 @@ def property(prop):
 def dump() -> str:
     return _MACRO.dump()
 
-def get_macro() -> str:
-    return str(_MACRO)
+def get_macro(mapping=None) -> str:
+    if mapping:
+        macro = _MACRO.format(mapping)
+    else:
+        macro = _MACRO
+    return str(macro)
+
+def set_flags(**flags: MacroFlagOptions):
+    """
+    Set macro flags.
+    """    
+    _MACRO._flags = MacroFlags(**flags)
+    return None
 
 del wraps
\ No newline at end of file
