@@ -1,72 +1,96 @@
 from __future__ import annotations
 from copy import deepcopy
-from typing import Callable, Iterable, Iterator, Any
+from typing import Callable, Iterable, Iterator, Any, Hashable, overload
 from enum import Enum
 from numbers import Number
 from .symbol import Symbol
         
 class Head(Enum):
-    getattr = "getattr"
-    getitem = "getitem"
-    del_    = "del"
-    call    = "call"
-    assign  = "assign"
-    kw      = "kw"
-    comment = "comment"
-    assert_ = "assert_"
-    binop   = "binop"
+    empty    = "empty"
+    getattr  = "getattr"
+    getitem  = "getitem"
+    del_     = "del"
+    call     = "call"
+    assign   = "assign"
+    kw       = "kw"
+    comment  = "comment"
+    assert_  = "assert"
+    binop    = "binop"
+    block    = "block"
+    function = "function"
+    return_  = "return"
+    if_      = "if"
+    elif_    = "elif"
+    for_     = "for"
+    annotate = "annotate"
 
-EXEC = (Head.assign, Head.assert_, Head.comment)
+EXEC = (Head.assign, Head.assert_, Head.comment, Head.block, Head.function, Head.return_,
+        Head.if_, Head.elif_, Head.for_)
 
-BINOP_MAP = {
-    "__add__": Symbol("+"),
-    "__sub__": Symbol("-"),
-    "__mul__": Symbol("*"),
-    "__div__": Symbol("/"),
-    "__eq__": Symbol("=="),
-    "__neq__": Symbol("!="),
-    "__gt__": Symbol(">"),
-    "__ge__": Symbol(">="),
-    "__lt__": Symbol("<"),
-    "__le__": Symbol("<="),
-    "__pow__": Symbol("**"),
-    "__matmul__": Symbol("@"),
-    "__floordiv__": Symbol("//"),
-    "__and__": Symbol("&"),
-    "__or__": Symbol("|"),
-    "__xor__": Symbol("^")
+def as_str(expr: Any, indent: int = 0):
+    if isinstance(expr, Expr):
+        return _STR_MAP[expr.head](expr, indent)
+    else:
+        return " "*indent + str(expr)
+
+def rm_par(s: str):
+    if s[0] == "(" and s[-1] == ")":
+        s = s[1:-1]
+    return s
+
+def sjoin(sep: str, iterable: Iterable[Any], indent: int = 0):
+    return sep.join(as_str(expr, indent) for expr in iterable)
+
+_STR_MAP: dict[Head, Callable[[Expr, int], str]] = {
+    Head.empty    : lambda e, i: "",
+    Head.getattr  : lambda e, i: f"{as_str(e.args[0], i)}.{as_str(e.args[1])}",
+    Head.getitem  : lambda e, i: f"{as_str(e.args[0], i)}[{as_str(e.args[1])}]",
+    Head.del_     : lambda e, i: " "*i + f"del {as_str(e.args[0])}",
+    Head.call     : lambda e, i: f"{as_str(e.args[0], i)}({sjoin(', ', e.args[1:])})",
+    Head.assign   : lambda e, i: f"{as_str(e.args[0], i)} = {e.args[1]}",
+    Head.kw       : lambda e, i: f"{as_str(e.args[0])}={as_str(e.args[1])}",
+    Head.assert_  : lambda e, i: " "*i + f"assert {as_str(e.args[0])}, {as_str(e.args[1])}".rstrip(", "),
+    Head.comment  : lambda e, i: " "*i + f"# {e.args[0]}",
+    Head.binop    : lambda e, i: " "*i + f"({as_str(e.args[1])} {as_str(e.args[0])} {as_str(e.args[2])})",
+    
+    Head.block    : lambda e, i: sjoin("\n", e.args, i),
+    Head.function : lambda e, i: " "*i + f"def {as_str(e.args[0])}:\n{as_str(e.args[1], i+4)}",
+    Head.return_  : lambda e, i: " "*i + f"return {sjoin(', ', e.args)}",
+    Head.if_      : lambda e, i: " "*i + f"if {rm_par(as_str(e.args[0]))}:\n{as_str(e.args[1], i+4)}\n" + \
+                                 " "*i + f"else:\n{as_str(e.args[2], i+4)}",
+    Head.elif_    : lambda e, i: " "*i + f"if {rm_par(as_str(e.args[0]))}:\n{as_str(e.args[1], i+4)}\n" + \
+                                 " "*i + f"else:\n{as_str(e.args[2], i+4)}",
+    Head.for_     : lambda e, i: " "*i + f"for {rm_par(as_str(e.args[0]))}:\n{as_str(e.args[1], i+4)}",
+    Head.annotate : lambda e, i: f"{as_str(e.args[0], i)}: {as_str(e.args[1])}"
 }
 
 class Expr:
     n: int = 0
-    
-    # a map of how to conver expression into string.
-    _map: dict[Head, Callable[[Expr], str]] = {
-        Head.getattr: lambda e: f"{e.args[0]}.{e.args[1]}",
-        Head.getitem: lambda e: f"{e.args[0]}[{e.args[1]}]",
-        Head.del_   : lambda e: f"del {e.args[0]}",
-        Head.call   : lambda e: f"{e.args[0]}({', '.join(map(str, e.args[1:]))})",
-        Head.assign : lambda e: f"{e.args[0]} = {e.args[1]}",
-        Head.kw     : lambda e: f"{e.args[0]}={e.args[1]}",
-        Head.assert_: lambda e: f"assert {e.args[0]}, {e.args[1]}".rstrip(", "),
-        Head.comment: lambda e: f"# {e.args[0]}",
-        Head.binop  : lambda e: f"({e.args[1]} {e.args[0]} {e.args[2]})"
-    }
-    
+        
     def __init__(self, head: Head, args: Iterable[Any]):
-        self.head = Head(head)
-        self.args = list(map(self.__class__.parse_object, args))
+        self._head = Head(head)
+        self._args = list(map(self.__class__.parse_object, args))
             
         self.number = self.__class__.n
         self.__class__.n += 1
     
+    @property
+    def head(self) -> Head:
+        return self._head
+    
+    @property
+    def args(self) -> list[Expr|Symbol]:
+        return self._args
+        
     def __repr__(self) -> str:
         s = str(self)
-        s = s.lstrip("(").rstrip(")")
+        if len(s) > 1:
+            s = rm_par(s)
+        s = s.replace("\n", "\n  ")
         return f":({s})"
     
     def __str__(self) -> str:
-        return self.__class__._map[self.head](self)
+        return as_str(self)
     
     def __eq__(self, expr: Expr|Symbol) -> bool:
         if isinstance(expr, self.__class__):
@@ -90,15 +114,36 @@ class Expr:
             out.append(f"{i:>{ind+2}}: {value}\n")
         return "".join(out)
     
-    def dump(self):
-        return self._dump()
+    def dump(self) -> str:
+        """
+        Dump expression into a tree.
+        """        
+        s = self._dump()
+        return s.rstrip("\n") + "\n"
         
-    def copy(self):
+    def copy(self) -> Expr:
+        # Always copy object deeply.
         return deepcopy(self)
     
     def eval(self, _globals: dict[Symbol|str, Any] = {}, _locals: dict = {}):
+        """
+        Evaluate or execute macro as an Python script.
+        
+        Either ``eval`` or ``exec`` is called for every expressions, as determined by its
+        header. Calling this function is much safer than those not-recommended usage of 
+        ``eval`` or ``exec``.
+        
+        Parameters
+        ----------
+        _globals : dict[Symbol, Any], optional
+            Mapping from global variable symbols to their values.
+        _locals : dict, optional
+            Updated variable namespace. Will be a mapping from symbols to values.
+            
+        """        
         _globals = {(sym.name if isinstance(sym, Symbol) else sym): v 
                     for sym, v in _globals.items()}
+        # TODO: Here should be some better ways to assign proper scope.
         if self.head in EXEC:
             return exec(str(self), _globals, _locals)
         else:
@@ -185,8 +230,37 @@ class Expr:
         if not yielded:
             yield self
     
+    @overload
+    def format(self, mapping: dict[Hashable, Symbol|Expr], inplace: bool = False) -> Expr:...
+        
+    @overload
+    def format(self, mapping: Iterable[tuple[Any, Symbol|Expr]], inplace: bool = False) -> Expr:...
+    
     def format(self, mapping: dict[Symbol, Symbol|Expr], inplace: bool = False) -> Expr:
-        mapping = check_format_mapping(mapping.items())
+        """
+        Format expressions in the macro.
+        
+        Just like formatting method of string, this function can replace certain symbols to
+        others. 
+        
+        Parameters
+        ----------
+        mapping : dict or iterable of tuples
+            Mapping from objects to symbols or expressions. Keys will be converted to symbol.
+            For instance, if you used ``arr``, a numpy.ndarray as an input of an macro-recordable
+            function, that input will appear like 'var0x1...'. By calling ``format([(arr, "X")])``
+            then 'var0x1...' will be substituted to 'X'.
+        inplace : bool, default is False
+            Expression will be overwritten if true.
+            
+        Returns
+        -------
+        Expression
+            Formatted expression.
+        """        
+        if isinstance(mapping, dict):
+            mapping = mapping.items()
+        mapping = check_format_mapping(mapping)
             
         if not inplace:
             self = self.copy()
@@ -208,10 +282,17 @@ class Expr:
 
 def check_format_mapping(mapping_list: Iterable[tuple[Any, Any]]) -> dict[Symbol, Symbol|Expr]:
     _dict: dict[Symbol, Symbol] = {}
-    for k, v in mapping_list:
+    for comp in mapping_list:
+        if len(comp) != 2:
+            raise ValueError("Wrong style of mapping list.")
+        k, v = comp
         if isinstance(v, Expr) and v.head in EXEC:
             raise ValueError("Cannot replace a symbol to a non-evaluable expression.")
-        _dict[symbol(k)] = symbol(v)
+        
+        if isinstance(v, str) and not isinstance(k, Symbol):
+            _dict[symbol(k)] = Symbol(v)
+        else:
+            _dict[symbol(k)] = symbol(v)
     return _dict
 
 
@@ -273,6 +354,8 @@ def symbol(obj: Any, constant: bool = True) -> Symbol:
         seq = f"{objtype.__name__}({obj.start}, {obj.stop}, {obj.step})"
     elif objtype in Symbol._type_map:
         seq = Symbol._type_map[objtype](obj)
+    elif hasattr(obj, "__name__"):
+        seq = Symbol(obj.__name__)
     else:
         for k, func in Symbol._type_map.items():
             if isinstance(obj, k):
