@@ -1,47 +1,119 @@
 from __future__ import annotations
 from contextlib import contextmanager
 from copy import deepcopy
+from collections.abc import MutableSequence
 from importlib import import_module
 from functools import partial, wraps
 import inspect
-from collections import UserList
-from typing import Callable, Iterable, Iterator, Any, Union, overload, TypeVar, Hashable
+from typing import Callable, Iterable, Iterator, Any, TypedDict, Union, overload, TypeVar, NamedTuple
 from types import ModuleType
 
-from .expression import BINOP_MAP, Head, Expr, symbol, EXEC, check_format_mapping
+from .ast import parse
+from .expression import Head, Expr, symbol, EXEC
 from .symbol import Symbol
 
-# types
-MetaCallable = Union[Callable[[Expr], Expr], Callable[[Expr, Any], Expr]]
-Recordable = Union[property, Callable, type, ModuleType]
-_property = property
-_O = TypeVar("_O")
-
 _NON_RECORDABLE = ("__new__", "__class__", "__repr__", "__getattribute__", "__dir__", 
-                   "__init_subclass__", "__subclasshook__")
+                   "__init_subclass__", "__subclasshook__", "__class_getitem__")
 
 _INHERITABLE = ("__module__", "__name__", "__qualname__", "__doc__", "__annotations__")
 
-class Macro(UserList):
+BINOP_MAP = {
+    "__add__": Symbol("+"),
+    "__sub__": Symbol("-"),
+    "__mul__": Symbol("*"),
+    "__div__": Symbol("/"),
+    "__eq__": Symbol("=="),
+    "__neq__": Symbol("!="),
+    "__gt__": Symbol(">"),
+    "__ge__": Symbol(">="),
+    "__lt__": Symbol("<"),
+    "__le__": Symbol("<="),
+    "__pow__": Symbol("**"),
+    "__matmul__": Symbol("@"),
+    "__floordiv__": Symbol("//"),
+    "__and__": Symbol("&"),
+    "__or__": Symbol("|"),
+    "__xor__": Symbol("^")
+}
+
+class MacroFlags(NamedTuple):
     """
-    List of expressions. A Macro object corresponds to a Python script.
+    This immutable struct gives the infomation of what kind of expressions will be
+    recorded in a macro instance
     """    
-    def __init__(self, iterable: Iterable = (), *, active: bool = True):
-        super().__init__(iterable)
-        self.active = active
+    Get: bool = True
+    Set: bool = True
+    Delete: bool = True
+    Return: bool = True
+
+# types
+
+MetaCallable = Union[Callable[[Expr], Expr], Callable[[Expr, Any], Expr]]
+Recordable = Union[property, Callable, type, ModuleType]
+_O = TypeVar("_O")
+_Class = TypeVar("_Class")
+
+class MacroFlagOptions(TypedDict):
+    Get: bool
+    Set: bool
+    Delete: bool
+    Return: bool
+
+# classes
+
+class Macro(Expr, MutableSequence[Expr]):
+    """
+    A special form of an expression with header "block".
+    This class behaves in a list-like way and specifically useful for macro recording.
+    """    
+    _FLAG_MAP = {
+        "__getitem__": "Get",
+        "__getattr__": "Get",
+        "__setitem__": "Set",
+        "__setattr__": "Set",
+        "__delitem__": "Delete",
+        "__delattr__": "Delete"
+    }
+    
+    def __init__(self, args: Iterable[Expr] = (), *, flags: MacroFlagOptions = {}):
+        super().__init__(head=Head.block, args=args)
+        self.active = True
+        self._callbacks = []
+        self._flags = MacroFlags(**flags)
+    
+    @property
+    def callbacks(self):
+        """
+        Callback functions when a new macro is recorded.
+        """        
+        return self._callbacks
+    
+    @property
+    def flags(self):
+        """
+        Flags that determine what kind of expression will be recorded.
+        """        
+        return self._flags
         
-    def append(self, expr: Expr):
-        if not isinstance(expr, Expr):
-            raise TypeError("Cannot append objects to Macro except for Expr objecs.")
-        return super().append(expr)
-    
-    def insert(self, key: int, expr: Expr):
-        if not isinstance(expr, Expr):
-            raise TypeError("Cannot insert objects to Macro except for Expr objecs.")
-        return super().insert(key, expr)
-    
-    def __str__(self) -> str:
-        return "\n".join(map(str, self))
+    def insert(self, key: int, expr: Expr | str):
+        """
+        Insert expressiong to macro.
+
+        Parameters
+        ----------
+        key : int
+            Position to insert.
+        expr : Expr
+            Expression. If a string is given, it will be parsed to Expr object.
+        """        
+        if isinstance(expr, str):
+            expr = parse(expr)
+        elif not isinstance(expr, Expr):
+            raise TypeError("Cannot insert objects to Macro except for Expr objects.")
+        
+        self.args.insert(key, expr)
+        for callback in self._callbacks:
+            callback(expr)
     
     @overload
     def __getitem__(self, key: int | str) -> Expr: ...
@@ -50,55 +122,19 @@ class Macro(UserList):
     def __getitem__(self, key: slice) -> Macro[Expr]: ...
         
     def __getitem__(self, key):
-        return super().__getitem__(key)
+        return self._args[key]
+    
+    def __setitem__(self, key: int, value: Expr):
+        self._args[key] = value
+    
+    def __delitem__(self, key: int):
+        del self._args[key]
+    
+    def __len__(self) -> int:
+        return len(self._args)
     
     def __iter__(self) -> Iterator[Expr]:
-        return super().__iter__()
-    
-    def __repr__(self) -> str:
-        return "\n".join(map(repr, self))
-    
-    def dump(self) -> str:
-        """
-        Make a meta-code in Julian way.
-
-        Returns
-        -------
-        str
-            Meta-code.
-        """        
-        return ",\n".join(expr.dump() for expr in self)
-    
-    def eval(self, _globals: dict[Symbol, Any] = {}, _locals: dict[Symbol, Any] = {}) -> Any:
-        """
-        Evaluate or execute macro as an Python script.
-        
-        Either ``eval`` or ``exec`` is called for every expressions, as determined by its
-        header. Calling this function is much safer than those not-recommended usage of 
-        ``eval`` or ``exec``.
-
-        Parameters
-        ----------
-        _globals : dict[Symbol, Any], optional
-            Mapping from global variable symbols to their values.
-        _locals : dict, optional
-            Updated variable namespace. Will be a mapping from symbols to values.
-
-        Returns
-        -------
-        Any
-            The last returned value.
-        """        
-        namespace = _globals.copy()
-        names = dict()
-        with self.blocked():
-            for expr in self:
-                namespace.update(names)
-                names = {}
-                out = expr.eval(namespace, names)
-            namespace.update(names)
-        _locals.update({Symbol(k): v for k, v in namespace.items()})
-        return out
+        return iter(self._args)
     
     @contextmanager
     def blocked(self):
@@ -113,58 +149,19 @@ class Macro(UserList):
             self.active = was_active
     
     @overload
-    def format(self, mapping: dict[Hashable, Symbol|Expr], inplace: bool = False) -> Macro:...
-        
-    @overload
-    def format(self, mapping: Iterable[tuple[Any, Symbol|Expr]], inplace: bool = False) -> Macro:...
-    
-    def format(self, mapping, inplace: bool = False) -> Macro:
-        """
-        Format expressions in the macro.
-        
-        Just like format method of string, this function can replace certain symbols to
-        others. 
-
-        Parameters
-        ----------
-        mapping : dict or iterable of tuples
-            Mapping from objects to symbols or expressions. Keys will be converted to symbol.
-            For instance, if you used ``arr``, a numpy.ndarray as an input of an macro-recordable
-            function, that input will appear like 'var0x1...'. By calling ``format([(arr, "X")])``
-            then 'var0x1...' will be substituted to 'X'.
-        inplace : bool, default is False
-            Macro will be overwritten if true.
-
-        Returns
-        -------
-        Macro
-            Formatted macro.
-        """        
-        if isinstance(mapping, dict):
-            mapping = mapping.items()
-            
-        m = check_format_mapping(mapping)
-        cls = self.__class__
-        
-        if inplace:
-            return cls(expr._unsafe_format(m) for expr in self)
-        else:
-            return cls(expr.copy()._unsafe_format(m) for expr in self)
-    
-    @overload
-    def record(self, obj: _property, *, returned_callback: MetaCallable = None) -> mProperty: ...
+    def record(self, obj: property, *, returned_callback: MetaCallable = None) -> mProperty: ...
     
     @overload
     def record(self, obj: ModuleType, *, returned_callback: MetaCallable = None) -> mModule: ...
     
     @overload
-    def record(self, obj: type, *, returned_callback: MetaCallable = None) -> type[MacroMixin]: ...
+    def record(self, obj: _Class, *, returned_callback: MetaCallable = None) -> _Class: ...
     
     @overload
     def record(self, obj: Callable, *, returned_callback: MetaCallable = None) -> mFunction: ...
     
     @overload
-    def record(self, *, returned_callback: MetaCallable = None) -> Callable[[Recordable], mObject|MacroMixin]: ...
+    def record(self, *, returned_callback: MetaCallable = None) -> Callable[[Recordable], mObject | MacroMixin]: ...
     
     def record(self, obj = None, *, returned_callback = None):
         """
@@ -178,31 +175,51 @@ class Macro(UserList):
             A function that will called after new expression is appended. Must take an expression
             or an expression with the last returned value as inputs.
         """        
+        kwargs = dict(macro=self, 
+                      returned_callback=returned_callback, 
+                      record_returned=self._flags.Return
+                      )
         def wrapper(_obj):
             if isinstance(_obj, property):
-                return mProperty(_obj, macro=self, returned_callback=returned_callback)
-            elif inspect.ismodule(_obj):
-                return mModule(_obj, macro=self, returned_callback=returned_callback)
-            elif inspect.isclass(_obj) and _obj is not type:
-                return self.record_methods(_obj, returned_callback=returned_callback)
-            elif callable(_obj) and not isinstance(_obj, mObject):
-                return mFunction(_obj, macro=self, returned_callback=returned_callback)
+                return mProperty(_obj, macro=self, returned_callback=returned_callback, 
+                                 flags=self.flags._asdict())
+            elif isinstance(_obj, ModuleType):
+                return mModule(_obj, **kwargs)
+            elif isinstance(_obj, type) and _obj is not type:
+                return self._record_methods(_obj, returned_callback=returned_callback)
+            elif isinstance(_obj, Callable) and not isinstance(_obj, mObject):
+                if not isclassmethod(_obj):
+                    return mFunction(_obj, **kwargs)
+                else:
+                    return mClassMethod(_obj, **kwargs)
+            elif isinstance(_obj, mObject):
+                return type(_obj)(_obj.obj, **kwargs)
             else:
                 raise TypeError(f"Type {type(_obj)} is not macro recordable.")
         
         return wrapper if obj is None else wrapper(obj)
 
     
-    def record_methods(self, cls: type, returned_callback: MetaCallable = None):
-        _dict = {}
+    def _record_methods(self, cls: type, returned_callback: MetaCallable = None):
+        _dict: dict[str, mObject] = {}
         for name, attr in inspect.getmembers(cls):
             if name in _NON_RECORDABLE:
                 continue
-            if callable(attr) or isinstance(attr, property):
+            if isinstance(attr, Callable):
+                key = self._FLAG_MAP.get(name, "None")
+                if not getattr(self._flags, key, True):
+                    continue
+                _dict[name] = self.record(attr, returned_callback=returned_callback)
+            if isinstance(attr, property):
                 _dict[name] = self.record(attr, returned_callback=returned_callback)
             elif isinstance(attr, MacroMixin):
-                update_namespace(attr)
+                update_namespace(attr, Symbol(cls))
                 _dict[name] = attr
+            else:
+                try:
+                    _dict[name] = self.record(attr, returned_callback=returned_callback)
+                except TypeError:
+                    pass
         
         newcls = type(cls.__name__, (MacroMixin, cls), _dict)
         
@@ -224,6 +241,25 @@ class Macro(UserList):
         """
         return self.record(property(prop))
     
+    def classmethod(self, method: Callable):
+        """
+        Make a macro-recordable property similar to ``@classmethod``.
+
+        Parameters
+        ----------
+        method : callable
+            Function that will be a classmethod.
+
+        Returns
+        -------
+        mClassMethod
+            Macro-recordable classmethod object.
+        """
+        return self.record(classmethod(method))
+    
+    def staticmethod(self, method: Callable):
+        return self.record(staticmethod(method))
+    
     def optimize(self, inplace=False) -> Macro:
         """
         Optimize macro readability by deleting unused variables.
@@ -240,7 +276,7 @@ class Macro(UserList):
         """        
         if not inplace:
             self = deepcopy(self)
-        expr_map: list[tuple[Symbol, Expr]] = []
+        expr_map: list[tuple[Symbol, int]] = []
         need = set()
         for i, expr in enumerate(self):
             expr: Expr|Symbol
@@ -262,8 +298,7 @@ class Macro(UserList):
                 self[i] = self[i].args[1]
                 
         return self
-        
-        
+            
     def call_function(self, func: Callable, *args, **kwargs):
         """
         Call function in macro recording mode.
@@ -277,15 +312,28 @@ class Macro(UserList):
             out = func(*args, **kwargs)
         if self.active:
             expr = Expr.parse_call(func, args, kwargs)
-            line = _assign_value_callback(expr, out)
-            self.append(line)
+            if self._flags.Return:
+                expr = _assign_value(expr, out)
+            self.append(expr)
             self._last_setval = None
         return out
 
 class MacroMixin:
-    pass
+    """
+    Any class objects that are wrapped with ``Macro.record`` will inherit this class.
+    """
 
 def update_namespace(obj: MacroMixin, namespace: Symbol | Expr) -> None:
+    """
+    Update the namespace of a ``MacroMixin`` object.
+
+    Parameters
+    ----------
+    obj : MacroMixin
+        Input object.
+    namespace : Symbol or Expr
+        Namespace.
+    """
     new = Expr(Head.getattr, [namespace, symbol(obj)])
     for name, attr in inspect.getmembers(obj):
         if isinstance(attr, mObject):
@@ -294,86 +342,112 @@ def update_namespace(obj: MacroMixin, namespace: Symbol | Expr) -> None:
             update_namespace(attr, new)
     
 class mObject:
+    """
+    Abstract class for macro recorder equipped objects.
+    """    
     obj: Any
     def __init__(self, obj: Any, macro: Macro, returned_callback: MetaCallable = None, 
-                 namespace: Symbol|Expr = None) -> None:
+                 namespace: Symbol|Expr = None, record_returned: bool = True) -> None:
         self.obj = obj
-        self.returned_callback = returned_callback or (lambda expr, out: expr)
+        
+        if returned_callback is not None:
+            _callback_nargs = len(inspect.signature(returned_callback).parameters)
+            if _callback_nargs == 1:
+                returned_callback = lambda expr, out: returned_callback(expr)
+            
+            elif _callback_nargs != 2:
+                raise TypeError("returned_callback cannot take arguments more than two.")
+            
+            if record_returned:
+                self.returned_callback = lambda expr, out: \
+                    returned_callback(_assign_value(expr, out), out)
+            else:
+                self.returned_callback = returned_callback
+        
+        else:
+            if record_returned:
+                self.returned_callback = _assign_value
+            else:
+                self.returned_callback = lambda expr, out: expr
+            
         self.namespace = namespace
-        _callback_nargs = len(inspect.signature(self.returned_callback).parameters)
-        if _callback_nargs == 1:
-            self.returned_callback = lambda expr, out: self.returned_callback(expr)
         
-        elif _callback_nargs != 2:
-            raise TypeError("returned_callback cannot take arguments more than two.")
-        
-        self.macro: Macro = macro
+        self.macro = macro
         for name in _INHERITABLE:
             if hasattr(self.obj, name):
                 setattr(self, name, getattr(self.obj, name))
         
-        self._last_setval: tuple[Head, Any] = None
+        self._last_setval: Expr = None
     
-    def to_namespace(self, obj) -> Symbol | Expr:
+    def to_namespace(self, obj: Any) -> Symbol | Expr:
+        """
+        Return the expression of ``obj`` in the correct name space.
+        """        
+        sym = symbol(obj)
         if self.namespace is None:
-            return symbol(obj)
+            return sym
         else:
-            return Expr(Head.getattr, [self.namespace, symbol(obj)])
+            return Expr(Head.getattr, [self.namespace, sym])
 
 Symbol.register_type(mObject, lambda o: symbol(o.obj))
 
-class mFunction(mObject):
+class mCallable(mObject):
     obj: Callable
     def __init__(self, function: Callable, macro: Macro, returned_callback: MetaCallable = None,
-                 namespace: Symbol|Expr = None):
-        super().__init__(function, macro, returned_callback, namespace)
-        self._method_type = self._make_method_type()
-            
+                 namespace: Symbol|Expr = None, record_returned: bool = True):
+        super().__init__(function, macro, returned_callback, namespace, record_returned)
+    
     @property
     def __signature__(self):
         if hasattr(self.obj, "__signature__"):
             return self.obj.__signature__
         else:
             return inspect.signature(self.obj)
-            
+    
     def __call__(self, *args, **kwargs):
         with self.macro.blocked():
             out = self.obj(*args, **kwargs)
         if self.macro.active:
             expr = Expr.parse_call(self.to_namespace(self.obj), args, kwargs)
-            expr = _assign_value_callback(expr, out)
             line = self.returned_callback(expr, out)
             self.macro.append(line)
             self._last_setval = None
         return out
     
+class mFunction(mCallable):
+    """
+    Macro recorder equipped functions. Generated functions are also compatible with methods.
+    """
+    def __init__(self, function: Callable, macro: Macro, returned_callback: MetaCallable = None,
+                 namespace: Symbol|Expr = None, record_returned: bool = True):
+        super().__init__(function, macro, returned_callback, namespace, record_returned)
+        self._method_type = self._make_method_type()
+            
     def _make_method_type(self):
+        # TODO: __get__ etc. is not recorded correctly
         fname = getattr(self.obj, "__name__", None)
         if fname == "__init__":
-            def make_expr(obj: _O, out, *args, **kwargs):
+            def make_expr(obj: _O, *args, **kwargs):
                 expr = Expr.parse_init(self.to_namespace(obj), obj.__class__, args, kwargs)
                 self._last_setval = None
                 return expr
         elif fname == "__call__":
-            def make_expr(obj: _O, out, *args, **kwargs):
+            def make_expr(obj: _O, *args, **kwargs):
                 expr = Expr.parse_call(self.to_namespace(obj), args, kwargs)
-                expr = _assign_value_callback(expr, out)
                 self._last_setval = None
                 return expr
         elif fname == "__getitem__":
-            def make_expr(obj: _O, out, *args):
+            def make_expr(obj: _O, *args):
                 expr = Expr(Head.getitem, [self.to_namespace(obj), args[0]])
-                expr = _assign_value_callback(expr, out)
                 self._last_setval = None
                 return expr
         elif fname == "__getattr__":
-            def make_expr(obj: _O, out, *args):
+            def make_expr(obj: _O, *args):
                 expr = Expr(Head.getattr, [self.to_namespace(obj), Symbol(args[0])])
-                expr = _assign_value_callback(expr, out)
                 self._last_setval = None
                 return expr
         elif fname == "__setitem__":
-            def make_expr(obj: _O, out, *args):
+            def make_expr(obj: _O, *args):
                 target = Expr(Head.getitem, [self.to_namespace(obj), args[0]])
                 expr = Expr(Head.assign, [target, args[1]])
                 if self._last_setval == target:
@@ -382,7 +456,7 @@ class mFunction(mObject):
                     self._last_setval = target
                 return expr
         elif fname == "__setattr__":
-            def make_expr(obj: _O, out, *args):
+            def make_expr(obj: _O, *args):
                 target = Expr(Head.getattr, [self.to_namespace(obj), Symbol(args[0])])
                 expr = Expr(Head.assign, [target, args[1]])
                 if self._last_setval == target:
@@ -391,27 +465,26 @@ class mFunction(mObject):
                     self._last_setval = target
                 return expr
         elif fname == "__delitem__":
-            def make_expr(obj: _O, out, *args):
+            def make_expr(obj: _O, *args):
                 target = Expr(Head.getitem, [self.to_namespace(obj), args[0]])
                 expr = Expr(Head.del_, [target])
                 self._last_setval = None
                 return expr
         elif fname == "__delattr__":
-            def make_expr(obj: _O, out, *args):
+            def make_expr(obj: _O, *args):
                 target = Expr(Head.getattr, [self.to_namespace(obj), args[0]])
                 expr = Expr(Head.del_, [target])
                 self._last_setval = None
                 return expr
         elif fname in BINOP_MAP.keys():
             op = BINOP_MAP[fname]
-            def make_expr(obj: _O, out, *args):
+            def make_expr(obj: _O, *args):
                 expr = Expr(Head.binop, [op, self.to_namespace(obj), args[0]])
                 self._last_setval = None
                 return expr
         else:
-            def make_expr(obj: _O, out, *args, **kwargs):
+            def make_expr(obj: _O, *args, **kwargs):
                 expr = Expr.parse_method(self.to_namespace(obj), self.obj, args, kwargs)
-                expr = _assign_value_callback(expr, out)
                 self._last_setval = None
                 return expr
         
@@ -420,7 +493,7 @@ class mFunction(mObject):
             with self.macro.blocked():
                 out = self.obj(obj, *args, **kwargs)
             if self.macro.active:
-                expr = make_expr(obj, out, *args, **kwargs)
+                expr = make_expr(obj, *args, **kwargs)
                 if expr is not None:
                     line = self.returned_callback(expr, out)
                 self.macro.append(line)
@@ -436,16 +509,59 @@ class mFunction(mObject):
         if obj is None:
             return self.obj
         else:
-            return partial(self._method_type, obj)
+            return mMethod(self._method_type, obj)
+
+class mMethod(partial, mCallable):
+    """
+    Macro recorder equipped method class. Instances are always built by the __get__ method
+    in mFunction instance.
+    """
+    pass
+
+class mClassMethod(mCallable): # TODO: inherit classmethod class if possible
+    """
+    Macro recorder equipped classmethod class.
+    """
+    def __init__(self, function: classmethod, macro: Macro, returned_callback: MetaCallable = None, 
+                 namespace: Symbol | Expr = None, record_returned: bool = True):
+        super().__init__(function, macro, returned_callback, namespace, record_returned)
+        self.__self__ = function.__self__
+        clsname = Symbol(self.__self__.__name__)
+        # TODO: check namespace in MacroMixin
+        if self.namespace is None:
+            self.namespace = clsname
+        else:
+            self.namespace = Expr(Head.getattr, [self.namespace, clsname])
+
+class mStaticMethod(mCallable):
+    """
+    Macro recorder equipped staticmethod class.
+    """
+    # TODO: How to determine a method is a staticmethod??
+    def __init__(self, function: staticmethod, macro: Macro, returned_callback: MetaCallable = None, 
+                 namespace: Symbol | Expr = None, record_returned: bool = True):
+        super().__init__(function, macro, returned_callback, namespace, record_returned)
         
+        clsname = function.__qualname__.split(".")[-2]
+        if self.namespace is None:
+            self.namespace = Symbol(clsname)
+        else:
+            self.namespace = Expr(Head.getattr, [self.namespace, clsname])
+            
 class mProperty(mObject, property):
+    """
+    Macro recorder equipped property class.
+    """
     obj: property
     def __init__(self, prop: property, macro: Macro, returned_callback: MetaCallable = None,
-                 namespace: Symbol|Expr = None):
-        super().__init__(prop, macro, returned_callback, namespace)
+                 namespace: Symbol|Expr = None, flags: MacroFlagOptions = {}):
+        self._flags = MacroFlags(**flags)
+        super().__init__(prop, macro, returned_callback, namespace, record_returned=self._flags.Return)
         self.obj = self.getter(prop.fget)
     
-    def getter(self, fget: Callable[[_O], None]):
+    def getter(self, fget: Callable[[_O], None]) -> property:
+        if not self._flags.Get:
+            return self.obj.getter(fget)
         key = Symbol(fget.__name__)
         @wraps(fget)
         def getter(obj):
@@ -453,25 +569,27 @@ class mProperty(mObject, property):
                 out = fget(obj)
             if self.macro.active:
                 expr = Expr(Head.getattr, [self.to_namespace(obj), key])
-                expr = _assign_value_callback(expr, out)
-                line = self.returned_callback(expr, out)
-                self.macro.append(line)
+                expr = self.returned_callback(expr, out)
+                self.macro.append(expr)
                 self._last_setval = None
             return out
         return self.obj.getter(getter)
         
-    def setter(self, fset: Callable[[_O, Any], None]):
+    def setter(self, fset: Callable[[_O, Any], None]) -> property:
+        if not self._flags.Set:
+            return self.obj.setter(fset)
         key = Symbol(fset.__name__)
         @wraps(fset)
         def setter(obj, value):
             with self.macro.blocked():
                 out = fset(obj, value)
             if self.macro.active:
-                expr = Expr(Head.setattr, [self.to_namespace(obj), key, value])
-                if self._last_setval == (Head.setattr, key.name):
+                target = Expr(Head.getattr, [self.to_namespace(obj), key])
+                expr = Expr(Head.assign, [target, value])
+                if self._last_setval == target:
                     self.macro.pop(-1)
                 else:
-                    self._last_setval = (Head.setattr, key.name)
+                    self._last_setval = target
                 
                 line = self.returned_callback(expr, out)
                 self.macro.append(line)
@@ -480,14 +598,17 @@ class mProperty(mObject, property):
         
         return self.obj.setter(setter)
     
-    def deleter(self, fdel: Callable[[_O], None]):
+    def deleter(self, fdel: Callable[[_O], None]) -> property:
+        if not self._flags.Delete:
+            return self.obj.deleter(fdel)
         key = Symbol(fdel.__name__)
         @wraps(fdel)
         def deleter(obj):
             with self.macro.blocked():
                 out = fdel(obj)
             if self.macro.active:
-                expr = Expr(Head.delattr, [self.to_namespace(obj), key])
+                target = Expr(Head.getattr, [self.to_namespace(obj), key])
+                expr = Expr(Head.del_, [target])
                 line = self.returned_callback(expr, out)
                 self.macro.append(line)
                 self._last_setval = None
@@ -508,6 +629,9 @@ class mProperty(mObject, property):
         return self.obj.__delete__(obj)
     
 class mModule(mObject):
+    """
+    Macro recorder equipped module class.
+    """
     obj: ModuleType
     def __getattr__(self, key: str):
         try:
@@ -535,18 +659,34 @@ class mModule(mObject):
                     expr = Expr.parse_init(out, cls, args, kwargs)    
                 else:
                     expr = Expr.parse_method(self.to_namespace(self.obj), attr, args, kwargs)
-                    expr = _assign_value_callback(expr, out)
-                line = self.returned_callback(expr, out)
-                self.macro.append(line)
+                expr = self.returned_callback(expr, out)
+                self.macro.append(expr)
                 self._last_setval = None
             return out
         return mfunc
     
 
-def _assign_value_callback(expr: Expr, out: Any):
+def _assign_value(expr: Expr, out: Any):
     out_sym = symbol(out, constant=False)
     if expr.head not in EXEC:
         expr_assign = Expr(Head.assign, [out_sym, expr])
     else:
         expr_assign = expr        
-    return expr_assign
\ No newline at end of file
+    return expr_assign
+
+def isclassmethod(method: Any):
+    """
+    Check if method is a class method.
+    
+    https://stackoverflow.com/questions/19227724/check-if-a-function-uses-classmethod
+    """    
+    bound_to = getattr(method, "__self__", None)
+    if not isinstance(bound_to, type):
+        # must be bound to a class
+        return False
+    name = method.__name__
+    for cls in bound_to.__mro__:
+        descriptor = vars(cls).get(name)
+        if descriptor is not None:
+            return isinstance(descriptor, classmethod)
+    return False
\ No newline at end of file
