@@ -1,21 +1,21 @@
 from __future__ import annotations
+from functools import wraps
 import inspect
-from enum import Enum
-from pathlib import Path
 from typing import Callable, Any, TypeVar
-from types import FunctionType, BuiltinFunctionType, ModuleType
+from types import FunctionType, BuiltinFunctionType, ModuleType, MethodType
 
 T = TypeVar("T")
 
+# TODO: method of making identical Symbols from string. Maybe using hash(str)?
+
 class Symbol:
     # Map of how to convert object into a symbol.
     _type_map: dict[type, Callable[[Any], str]] = {
         type: lambda e: e.__name__,
         FunctionType: lambda e: e.__name__,
         BuiltinFunctionType: lambda e: e.__name__,
+        MethodType: lambda e: e.__name__,
         ModuleType: lambda e: e.__name__.split(".")[-1],
-        Enum: lambda e: repr(str(e.name)),
-        Path: lambda e: f"r'{e}'",
         type(None): lambda e: "None",
     }
     
@@ -55,6 +55,19 @@ class Symbol:
             raise TypeError(f"'==' is not supported between Symbol and {type(other)}")
         return self.object_id == other.object_id
     
+    @classmethod
+    def var(cls, identifier: str, type: type):
+        
+        # TODO: Need tests.
+        # _variables: set[int] -> dict[str, Symbol]
+        # constant -> property
+        if not identifier.isidentifier():
+            raise ValueError(f"'{identifier}' is not a valid identifier.")
+        self = cls(identifier, 0, type)
+        self.object_id = id(self)
+        self.constant = False
+        return self
+    
     def as_parameter(self, default=inspect._empty):
         return inspect.Parameter(self._name, 
                                  inspect.Parameter.POSITIONAL_OR_KEYWORD, 
@@ -70,11 +83,26 @@ class Symbol:
     
     @classmethod
     def register_type(cls, type: type[T], function: Callable[[T], str|Symbol]):
+        """
+        Dispatch value into string in a certain rule.
+        
+        .. code-block:: python
+        
+            register_type(np.ndarray, lambda arr: str(arr.tolist()))
+
+        Parameters
+        ----------
+        type : type
+            Type of value.
+        function : callable
+            Conversion rule
+        """    
         if not callable(function):
             raise TypeError("The second argument must be callable.")
         cls._type_map[type] = function
         
-
+@wraps(Symbol.register_type)
 def register_type(type: type[T], function: Callable[[T], str|Symbol]):
     return Symbol.register_type(type, function)
-        
\ No newline at end of file
+
+del wraps
\ No newline at end of file
